### 动态模板渲染文档

### 高级素材替换说明

+ 设计师按以下规则，标记需要绑定附加素材的 ui_key

    如有一张图片素材需要关联一张附加素材，
    需要设计师把图片素材的 ui_key 标记为 dimg1，再把关联的附加素材标记为 dtext1.1, 引擎会通过 dimg1 和 dtext1.1 关联替换  
    如需要对应多张文字素材，依次设置附加素材的 ui_key 为 dtext1.2, dtext1.3

+ 通过 VeProcessRenderTask.setDynamicSubFiles 设置附加素材数组

    参数为 json 数组:

    ```json
    [
        {
            "img_path" : "./workspace/assets/2.jpeg", // 引擎会根据图片实际路径找到对应的图层或合层，找到对应的 ui_key 比如 dimg1
            "d_key_prefix" : "dtext", // 通过 dimg1 得到序号 1, 使用d_key_prefix的值，
                                      // 按下文的 d_img_paths 依次构造出 dtext1.1, dtext1.2 
            "d_img_paths" : [
                "./workspace/assets/235_41_text2.png", // 会替换  ui_key = dtext1.1 的图层
                "./workspace/assets/235_41_text3.png"  // 会替换 ui_key = dtext1.2 的图层
            ]
        }
    ]

    ```
    
    + 字段含义

            img_path      主替换图片的路径，用于找到对应的主素材合层
            d_key_prefix  附加素材图层　ui_key 的前缀或者全称  
            d_img_paths   附加素材路径，用于依次替换 ui_key 以 d_key_prefix 为前缀的附加素材图层

    + 示例数据说明

            以上数据示例有两个附加素材链接，假如 img_path 对应的 ui_key 是 dimg1, 则通过 dtext1 关联的附加素材. 
            会被依次替换：dtext1.1 会被替换成 d_img_paths 中的第一长图片, dtext1.2 会被替换成 d_img_paths 的第二张图片，以此类推

    
    + 关联格式

            img_path 对应素材的 ui_key 必须为 dimg 开头， 格式为 dimg{#}， {#}代表整数序号，比如 dimg1, dimg2。
            d_key_prefix 的 ui_key 格式为 dimg{#}.{#}， 第1个{#}对应 dimg 的序号， 第2个{#}对应附加素材的序号。
            比如 dtext1.1 为 dimg1 第1个附加素材，dtext1.2 则对应 dimg1 的第2个附加素材

    + 前缀查找替换数据格式
    
    ```json 
    [
        {
            "img_path" : "./workspace/assets/2.jpeg",  // 必须设置为有效路径, 前缀查找替换
            "d_key_prefix" : "dtext",                  // 前缀匹配，按 ui_key 以 dtext 开头依次替换
            "d_img_paths" : [                          // 模板支持的附加素材数目请咨询对应模板设计师 
                "./workspace/assets/235_41_text2.png",
                "./workspace/assets/235_41_text3.png"
            ]
        }
    ]
    
    ```
    
    + 精确匹配替换格式
    
    ```json 
    [
        {
            "img_path" : "",                // 设置为空字符串, 精确匹配替换
            "d_key_prefix" : "dheadimg",    // 精确匹配, ui_key = dheadimg 时替换
            "d_img_paths" : [
                "./workspace/assets/235_41_text9.png"
             ]
        }
    ]
    ```
        
    + 同时设置多个附加素材数据
        
    ```json 
    [
        {                   
            "img_path" : "",                        // 精确替换
            "d_key_prefix" : "dheadimg",   
            "d_img_paths" : [
                "./workspace/assets/235_41_text9.png"
             ]
        },
        {
            "img_path" : "./workspace/assets/2.jpeg",  // 前缀替换
            "d_key_prefix" : "dtext",                  
            "d_img_paths" : [                         
                "./workspace/assets/235_41_text2.png",
                "./workspace/assets/235_41_text3.png"
            ]
        },
        {
            "img_path" : "./workspace/assets/3.jpeg",  // 前缀替换
            "d_key_prefix" : "dtext",                  
            "d_img_paths" : [                           
                "./workspace/assets/235_41_text4.png",
            ]
        }
    ]
    ```


### 动态模板附加文字设置

动态模板支持附加文字的设置, 设计师需要按上文 "附加素材替换说明" 中的步骤编辑导出模板. 

*注意，标记附加文字素材需要保证为文字图层，才能导出文字信息供引擎绘制文字，如果附加素材对应的素材信息不是文字，会被忽略*

*文字绘制的效果由模板中的对应的文字配置信息决定, 引擎最终会调用 TextPainter 把文字转成图片参与渲染.*





+ 注意事项:

    + 字体请参考 textpainter 文件夹中的说明文档
    + 不支持 emoji
    + 必须调用 VeProcessRenderTask.setAssetDir 设置文字图片的保存目录, 不设置文字绘制无效
    + 必须调用 VeProcessRenderTask.setTextPainterDir 设置文字绘制工具目录，不设置文字绘制无效

        引擎实际调用 /path/to/TextPainter /path/to/font_list.json /path/to/text_list.json 命令去绘制图片。
        而上文设置的 AssetDir 用来保存生成的 text_list.json 和 文字图片,所以需要调用方渲染后清理素材。
    
    + *如发现文字图片没生成，请检查 TextPainter 的执行权限, font_list.json 中的字体路径，
        以及文字图层对应的字体是否在 font_list.json 中*    
    
    
+ 通过 VeProcessRenderTask.setDynamicSubTexts 设置附加文字素材数组
    参数为 json 数组:

    ```json
    [
        {
            "img_path" : "./workspace/assets/2.jpeg", // 引擎会根据图片实际路径找到对应的图层或合层，找到对应的 ui_key 比如 dimg1
            "d_key_prefix" : "dtext", // 通过 dimg1 得到序号 1, 使用d_key_prefix的值，
                                      // 按下文的 d_texts 依次构造出 dtext1.1, dtext1.2 
            "d_texts" : [
                "文字内容", 　// 会替换  ui_key = dtext1.1 的图层
                "文字内容"  // 会替换 ui_key = dtext1.2 的图层
            ]
        }
    ]

    ```
    
    + 字段含义

            img_path      主替换图片的路径，用于找到对应的主素材合层
            d_key_prefix  附加素材图层　ui_key 的前缀或者全称  
            d_texts  附加素材路径，用于依次替换 ui_key 以 d_key_prefix 为前缀的附加素材图层

    + 示例数据说明

            以上数据示例有两个附加素材链接，假如 img_path 对应的 ui_key 是 dimg1, 则通过 dtext1 关联的附加素材. 
            会被依次替换：dtext1.1 会被替换成 d_texts 中的第一张文字图片, dtext1.2 会被替换成 d_texts 的第二张文字图片，
            以此类推

    
    + 关联格式

            img_path 对应素材的 ui_key 必须为 dimg 开头， 格式为 dimg{#}， {#}代表整数序号，比如 dimg1, dimg2。
            d_key_prefix 的 ui_key 格式为 dimg{#}.{#}， 第1个{#}对应 dimg 的序号， 第2个{#}对应附加素材的序号。
            比如 dtext1.1 为 dimg1 第1个附加素材，dtext1.2 则对应 dimg1 的第2个附加素材


    + 前缀查找替换数据格式
    
    ```json 
    [
        {
            "img_path" : "./workspace/assets/2.jpeg",  // 必须设置为有效路径, 前缀查找替换
            "d_key_prefix" : "dtext",                  // 前缀匹配，按 ui_key 以 dtext 开头依次替换
            "d_texts" : [                              // 模板支持的附加文字数目请咨询对应模板设计师 
                "hello world",
                "hello world"
            ]
        }
    ]
    
    ```
    
    + 精确匹配替换格式
    
    ```json 
    [
        {
            "img_path" : "",                // 设置为空字符串, 精确匹配替换
            "d_key_prefix" : "nickname",    // 精确匹配, ui_key = dheadimg 时替换
            "d_texts" : [
                "用户昵称"
             ]
        }
    ]
    ```
        
    + 同时设置多个附加素材数据
        
    ```json 
    [
        {
            "img_path" : "",                // 精确替换
            "d_key_prefix" : "nickname",    
            "d_texts" : [
                "用户昵称"
             ]
        },
        {
            "img_path" : "./workspace/assets/2.jpeg",   // 前缀替换
            "d_key_prefix" : "dtext",                  
            "d_texts" : [                           
                "hello world",
                "hello ve text"
            ]
        },
        {
            "img_path" : "./workspace/assets/3.jpeg",  // 前缀替换
            "d_key_prefix" : "dtext",                 
            "d_texts" : [                          
                "hello dtext",
            ]
        }
    ]
    ```

+ 测试程序

    com.seeshiontech.vesdk.DynamicSubAssetTest.testDynamicText()